

library ieee ;
use ieee . std_logic_1164 . all ;

entity seven_segment_driver is
    port ( clock : in std_logic ;
           byte_to_output : in std_logic_vector ( 7 downto 0 ) ;
           blank_display : in std_logic ;
           enable : in std_logic ;
           led_output_pins : out std_logic_vector ( 6 to 19 ) ) ;
end entity seven_segment_driver ;


architecture basic of seven_segment_driver is
    subtype ad is std_logic_vector ( 1 to 4 ) ;
    subtype dt is std_logic_vector ( 1 to 7 ) ;
begin
    ssd_process : for digit in 0 to 1 generate
        process ( byte_to_output , clock , blank_display , enable ) is
            variable uout : dt ;
            variable nibble : ad ;
        begin
            if ( enable = '1' )
            and ( clock = '1' )
            and ( clock'event )
            then
                if ( blank_display = '1' )
                then
                    uout := dt'("0000000" ) ;
                else
                    nibble := byte_to_output( 
                        (( digit * 4 ) + 3 ) downto ( digit * 4 ) ) ;
                    case nibble is
                                    --                   edcgfab
                        when ad'("0000") => uout := dt'("1110111") ; -- 0
                        when ad'("0001") => uout := dt'("0010001") ;
                        when ad'("0010") => uout := dt'("1101011") ;
                        when ad'("0011") => uout := dt'("0111011") ;
                        when ad'("0100") => uout := dt'("0011101") ; -- 4
                        when ad'("0101") => uout := dt'("0111110") ;
                        when ad'("0110") => uout := dt'("1111110") ;
                        when ad'("0111") => uout := dt'("0010011") ;
                        when ad'("1000") => uout := dt'("1111111") ; -- 8
                        when ad'("1001") => uout := dt'("0011111") ;
                        when ad'("1010") => uout := dt'("1011111") ;
                        when ad'("1011") => uout := dt'("1111100") ;
                        when ad'("1100") => uout := dt'("1101000") ; -- c
                        when ad'("1101") => uout := dt'("1111001") ;
                        when ad'("1110") => uout := dt'("1101110") ;
                        when others =>      uout := dt'("1001110") ; -- f
                    end case ;
                end if ;
                if ( digit = 0 )
                then
                    -- less significant digit, i.e. digit on the right
                    led_output_pins ( 13 to 19 ) <= uout ;
                else 
                    -- more significant digit, i.e. digit on the left
                    led_output_pins ( 6 to 12 ) <= uout ;
                end if ;
            end if ;
        end process ;
    end generate ;

-- pin arrangements:
-- e1 - 6  -- e2 - 13
-- d1 - 7  -- d2 - 14
-- c1 - 8  -- c2 - 15
-- g1 - 9  -- g2 - 16
-- f1 - 10 -- f2 - 17
-- a1 - 11 -- a2 - 18
-- b1 - 12 -- b2 - 19


end architecture basic ;

