
    

    data_register_file : entity dp_ram
        generic map ( address_width => 3 ,      -- 8 x 32
                    data_width => 32 )
        port map ( clock => clock ,
                    write => reg_update_data_x ,
                    address1 => register_file_address_x ,
                    address2 => register_file_address_y ,
                    data_in => data_regs_in ,
                    data_out1 => register_file_data_out_x ,
                    data_out2 => register_file_data_out_y ) ;

    address_register_file : entity dp_ram
        generic map ( address_width => 3 ,      -- 8 x 32
                    data_width => 32 )
        port map ( clock => clock ,
                    write => reg_update_address_x ,
                    address1 => register_file_address_x ,
                    address2 => register_file_address_y ,
                    data_in => address_regs_in ,
                    data_out1 => register_file_address_out_x ,
                    data_out2 => register_file_address_out_y ) ;


    -- The register numbers, x and y, are chosen here. #
    register_file_source_mux : process ( register_file_source_x ,
                    register_file_source_y ,
                    instruction_register , ea_reg ) is
    begin
        case register_file_source_x is
        when RF_X_EA_REG =>
                register_file_address_x <= ea_reg ;
        when RF_X_FORCE_TO_SP =>
                register_file_address_x <= "111" ;
        when RF_X_11_TO_9_FIELD =>
                register_file_address_x <= 
                            instruction_register ( 11 downto 9 ) ;
        when others => -- RF_X_2_TO_0_FIELD =>
                register_file_address_x <= 
                            instruction_register ( 2 downto 0 ) ;
        end case ;
        
        case register_file_source_y is
        when RF_Y_FORCE_TO_SP =>
                register_file_address_y <= "111" ;
        when RF_Y_11_TO_9_FIELD =>
                register_file_address_y <= 
                            instruction_register ( 11 downto 9 ) ;
        when others => -- RF_Y_2_TO_0_FIELD =>
                register_file_address_y <=
                            instruction_register ( 2 downto 0 ) ;
        end case ;
    end process register_file_source_mux ;

    -- The input to the register file is basically alu_output. However,
    -- if a single word is written to a register, the high word is unchanged.
    -- The following code supports this
    register_file_input : process ( alu_output , operation_size ,
                    reg_update_override_size , 
                    register_file_address_out_x ,
                    register_file_data_out_x ) is
    begin
        if ( reg_update_override_size = '1' )
        or ( operation_size = DWORD )
        then
            -- Treat as a DWORD
            address_regs_in <= alu_output ;
            data_regs_in <= alu_output ;
        else
            if ( operation_size = BYTE )
            then
                address_regs_in ( 31 downto 8 ) <= 
                                ( others => alu_output ( 7 ) ) ;
                address_regs_in ( 7 downto 0 ) <= 
                                alu_output ( 7 downto 0 ) ;
                data_regs_in ( 31 downto 8 ) <= 
                                ( others => alu_output ( 7 ) ) ;
                data_regs_in ( 7 downto 0 ) <= 
                                alu_output ( 7 downto 0 ) ;
            else
                address_regs_in ( 31 downto 16 ) <= 
                                ( others => alu_output ( 15 ) ) ;
                address_regs_in ( 15 downto 0 ) <= 
                                alu_output ( 15 downto 0 ) ;
                data_regs_in ( 31 downto 16 ) <= 
                                ( others => alu_output ( 15 ) ) ;
                data_regs_in ( 15 downto 0 ) <= 
                                alu_output ( 15 downto 0 ) ;
            end if ;
        end if ;
    end process register_file_input ;

